//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  DeepSkyBlue
#property indicator_color2  PaleVioletRed
#property indicator_color3  Silver
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern string TimeFrame1         = "Current time frame";
extern string TimeFrame2         = "Next time frame";
extern int    RsiPeriod          = 14;
extern int    RsiPrice           = PRICE_CLOSE;
extern int    RsiSmoothingFactor = 5;
extern int    RsiMethod          = 1;
extern int    MaPeriod           = 0;
extern int    MaMethod           = MODE_LWMA;
extern double WP                 = 4.236;

//
//
//
//
//

double qqeUp[];
double qqeDn[];
double qqeNe[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame1;
int    timeFrame2;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,qqeUp); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,qqeDn); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,qqeNe); SetIndexStyle(2,DRAW_HISTOGRAM);


   //
   //
   //
   //
   //
   
      MaPeriod = MathMax(MaPeriod,1);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame1=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame1=="calculateValue"; if (calculateValue) return(0);
      timeFrame1        = stringToTimeFrame(TimeFrame1);
         if (StringFind(StringUpperCase(TimeFrame2),"NEXT")>-1)
               timeFrame2 = nextTimeFrame(timeFrame1);
         else  timeFrame2 = stringToTimeFrame(TimeFrame2);

   //
   //
   //
   //
   //
         
   IndicatorShortName("QQE of "+getRsiName(RsiMethod)+" combined "+timeFrameToString(timeFrame1)+" - "+timeFrameToString(timeFrame2)+" ("+RsiPeriod+","+RsiSmoothingFactor+")");
   return(0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double work[][5];
#define _ema1  0
#define _ema2  1
#define _tr    2
#define _rsi   3
#define _trend 4


//
//
//
//
//

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = Bars-counted_bars;
           if (returnBars) { qqeUp[0] = limit+1; return(0); }
           if (ArrayRange(work,0) != Bars) ArrayResize(work,Bars);

   //
   //
   //
   //
   //

   if (calculateValue)
   {
      double alpha1 = 2.0/(RsiSmoothingFactor+1.0);
      double alpha2 = 1.0/(RsiPeriod);
      for (i=limit, r=Bars-i-1; i>=0; i--,r++)
      {  
         work[r][_rsi]  = work[r-1][_rsi]  + alpha1*(iRsi(iMA(NULL,0,MaPeriod,0,MaMethod,RsiPrice,i),RsiPeriod,i,0,RsiMethod) - work[r-1][_rsi] );
         work[r][_ema1] = work[r-1][_ema1] + alpha2*(MathAbs(work[r-1][_rsi]-work[r][_rsi])                                   - work[r-1][_ema1]);
         work[r][_ema2] = work[r-1][_ema2] + alpha2*(work[r][_ema1]                                                           - work[r-1][_ema2]);

         //
         //
         //
         //
         //

         double dar = work[r  ][_ema2]*WP;
         double tr  = work[r-1][_tr];
         double dv  = tr;
   
            if (work[r][_rsi]   < tr) { tr = work[r][_rsi] + dar; if ((work[r-1][_rsi] < dv) && (tr > dv)) tr = dv; }
            if (work[r][_rsi]   > tr) { tr = work[r][_rsi] - dar; if ((work[r-1][_rsi] > dv) && (tr < dv)) tr = dv; }
                work[r][_tr]    = tr;
                work[r][_trend] = work[r-1][_trend];
         
                if (work[r][_tr]<work[r][_rsi]) work[r][_trend] =  1;
                if (work[r][_tr]>work[r][_rsi]) work[r][_trend] = -1;
                qqeUp[i] = work[r][_trend];
      }    
      return(0);
   }      

   //
   //
   //
   //
   //

   if (timeFrame1 != Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame1,indicatorFileName,"returnBars",0,0)*timeFrame1/Period()));
   if (timeFrame2 != Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame2,indicatorFileName,"returnBars",0,0)*timeFrame2/Period()));
   for(i=limit; i>=0; i--)
   {
      double trend  = iCustom(NULL,timeFrame1,indicatorFileName,"calculateValue","",RsiPeriod,RsiPrice,RsiSmoothingFactor,RsiMethod,MaPeriod,MaMethod,WP,0,iBarShift(NULL,timeFrame1,Time[i]));
             trend += iCustom(NULL,timeFrame2,indicatorFileName,"calculateValue","",RsiPeriod,RsiPrice,RsiSmoothingFactor,RsiMethod,MaPeriod,MaMethod,WP,0,iBarShift(NULL,timeFrame2,Time[i]));
         
         //
         //
         //
         //
         //
         
         qqeUp[i] = EMPTY_VALUE;
         qqeDn[i] = EMPTY_VALUE;
         qqeNe[i] = EMPTY_VALUE;
            if (trend>0) { qqeUp[i] = 1; continue; }
            if (trend<0) { qqeDn[i] = 1; continue; }
                           qqeNe[i] = 1;
   }       
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int nextTimeFrame(int tf)
{
   double max = ArraySize(iTfTable)-1;
   int    ret = -1;
   
      //
      //
      //
      //
      //
      
      for (int i=max; i>=0 && tf!=iTfTable[i]; i--) ret = iTfTable[i];
   return(MathMax(ret,tf));
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//
//

string rsiMethodNames[] = {"rsi","Wilders rsi","rsx","Cuttler RSI"};
string getRsiName(int& method)
{
   int max = ArraySize(rsiMethodNames)-1;
      method=MathMax(MathMin(method,max),0); return(rsiMethodNames[method]);
}

//
//
//
//
//

double workRsi[][13];
#define _price  0
#define _change 1
#define _changa 2

double iRsi(double price, double period, int i, int instanceNo=0, int rsiMode=0)
{
   if (ArrayRange(workRsi,0)!=Bars) ArrayResize(workRsi,Bars);
      int z = instanceNo*13; 
      int r = Bars-i-1;
   
   //
   //
   //
   //
   //
   
   workRsi[r][z+_price] = price;
   switch (rsiMode)
   {
      case 0:
         double alpha = 1.0/period; 
         if (r<period)
            {
               int k; double sum = 0; for (k=0; k<period && (r-k-1)>=0; k++) sum += MathAbs(workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price]);
                  workRsi[r][z+_change] = (workRsi[r][z+_price]-workRsi[0][z+_price])/MathMax(k,1);
                  workRsi[r][z+_changa] =                                         sum/MathMax(k,1);
            }
         else
            {
               double change = workRsi[r][z+_price]-workRsi[r-1][z+_price];
                               workRsi[r][z+_change] = workRsi[r-1][z+_change] + alpha*(        change  - workRsi[r-1][z+_change]);
                               workRsi[r][z+_changa] = workRsi[r-1][z+_changa] + alpha*(MathAbs(change) - workRsi[r-1][z+_changa]);
            }
         if (workRsi[r][z+_changa] != 0)
               return(50.0*(workRsi[r][z+_change]/workRsi[r][z+_changa]+1));
         else  return(50.0);
         
      //
      //
      //
      //
      //
      
      case 1 :
         workRsi[r][z+1] = iSmma(0.5*(MathAbs(workRsi[r][z+_price]-workRsi[r-1][z+_price])+(workRsi[r][z+_price]-workRsi[r-1][z+_price])),0.5*(period-1),Bars-i-1,0);
         workRsi[r][z+2] = iSmma(0.5*(MathAbs(workRsi[r][z+_price]-workRsi[r-1][z+_price])-(workRsi[r][z+_price]-workRsi[r-1][z+_price])),0.5*(period-1),Bars-i-1,1);
         if((workRsi[r][z+1] + workRsi[r][z+2]) != 0) 
               return(100.0 * workRsi[r][z+1]/(workRsi[r][z+1] + workRsi[r][z+2]));
         else  return(50);

      //
      //
      //
      //
      //

      case 2 :     
         double Kg = (3.0)/(2.0+period), Hg = 1.0-Kg;
         if (r<period) { for (k=1; k<13; k++) workRsi[r][k+z] = 0; return(50); }  

         //
         //
         //
         //
         //
      
         double mom = workRsi[r][_price+z]-workRsi[r-1][_price+z];
         double moa = MathAbs(mom);
         for (k=0; k<3; k++)
         {
            int kk = k*2;
            workRsi[r][z+kk+1] = Kg*mom                + Hg*workRsi[r-1][z+kk+1];
            workRsi[r][z+kk+2] = Kg*workRsi[r][z+kk+1] + Hg*workRsi[r-1][z+kk+2]; mom = 1.5*workRsi[r][z+kk+1] - 0.5 * workRsi[r][z+kk+2];
            workRsi[r][z+kk+7] = Kg*moa                + Hg*workRsi[r-1][z+kk+7];
            workRsi[r][z+kk+8] = Kg*workRsi[r][z+kk+7] + Hg*workRsi[r-1][z+kk+8]; moa = 1.5*workRsi[r][z+kk+7] - 0.5 * workRsi[r][z+kk+8];
         }
         if (moa != 0)
              return(MathMax(MathMin((mom/moa+1.0)*50.0,100.00),0.00)); 
         else return(50);
            
      //
      //
      //
      //
      //
      
      case 3 :
         double sump = 0;
         double sumn = 0;
         for (k=0; k<period; k++)
         {
            double diff = workRsi[r-k][z+_price]-workRsi[r-k-1][z+_price];
               if (diff > 0) sump += diff;
               if (diff < 0) sumn -= diff;
         }
         if (sumn > 0)
               return(100.0-100.0/(1.0+sump/sumn));
         else  return(50);
   } 
   return(0);
}

//
//
//
//
//
//

double workSmma[][2];
double iSmma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= Bars) ArrayResize(workSmma,Bars);

   //
   //
   //
   //
   //

   if (r<period)
         workSmma[r][instanceNo] = price;
   else  workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}